/*******************************************************************************************************
****  Copyright (c) 2012 CA.  All rights reserved.  
****  This software and all information contained therein is confidential and proprietary and shall 
****  not be duplicated, used, disclosed or disseminated in any way except as authorized by the 
****  applicable license agreement, without the express written permission of CA. All authorized 
****  reproductions must be marked with this language.  
****  
****  EXCEPT AS SET FORTH IN THE APPLICABLE LICENSE AGREEMENT, TO THE EXTENT PERMITTED BY APPLICABLE 
****  LAW, CA PROVIDES THIS SOFTWARE WITHOUT WARRANTY OF ANY KIND, INCLUDING WITHOUT LIMITATION, ANY 
****  IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT WILL CA 
****  BE LIABLE TO THE END USER OR ANY THIRD PARTY FOR ANY LOSS OR DAMAGE, DIRECT OR INDIRECT, FROM 
****  THE USE OF THIS SOFTWARE, INCLUDING WITHOUT LIMITATION, LOST PROFITS, BUSINESS INTERRUPTION, 
****  GOODWILL, OR LOST DATA, EVEN IF CA IS EXPRESSLY ADVISED OF SUCH LOSS OR DAMAGE.
****  
****  
********************************************************************************************************/

/*
  This script creates an index for every foreign key in the schema if it does not already exist.
  The script will take a while to complete for a large database.  It is recommended that the script be
  executed during off hours to avoid performance degrade.  Also ensure there is adequate free tablespace
  space and disk space for the new indexes.

  Usage:
  1.  Log on to SQL*Plus as nolio user
  2.  Execute the sql file
        @create_FK_indexes_ora.sql
*/

COLUMN CUR_USER FORMAT A30
COLUMN VERSION_NAME FORMAT A30
SET SERVEROUTPUT ON


select sys_context('USERENV', 'CURRENT_SCHEMA') CUR_USER, v.VERSION_NAME from version_table v;

DECLARE
  cursor c1 is
  select  'CREATE INDEX '||substr(constraint_name,1,28)||'_I ON '||table_name||'('||
  cname1 || nvl2(cname2,','||cname2,null) ||
  nvl2(cname3,','||cname3,null) || nvl2(cname4,','||cname4,null) ||
  nvl2(cname5,','||cname5,null) || nvl2(cname6,','||cname6,null) ||
  nvl2(cname7,','||cname7,null) || nvl2(cname8,','||cname8,null)||')'  stmt
  from ( select b.table_name,
  b.constraint_name,
  max(decode( position, 1, column_name, null )) cname1,
  max(decode( position, 2, column_name, null )) cname2,
  max(decode( position, 3, column_name, null )) cname3,
  max(decode( position, 4, column_name, null )) cname4,
  max(decode( position, 5, column_name, null )) cname5,
  max(decode( position, 6, column_name, null )) cname6,
  max(decode( position, 7, column_name, null )) cname7,
  max(decode( position, 8, column_name, null )) cname8,
  c.table_name parent_table,
  count(*) col_cnt
  from (select substr(table_name,1,30) table_name,
  substr(constraint_name,1,30) constraint_name,
  substr(column_name,1,30) column_name,
  position
  from user_cons_columns ) a,
  user_constraints b, user_constraints c
  where a.constraint_name = b.constraint_name
  and b.constraint_type = 'R'
  and c.constraint_name = b.R_CONSTRAINT_NAME
  group by b.table_name, b.constraint_name, c.table_name
  ) cons
  where col_cnt > ALL
  ( select count(*)
  from user_ind_columns i
  where i.table_name = cons.table_name
  and i.column_name in (cname1, cname2, cname3, cname4,
  cname5, cname6, cname7, cname8 )
  and i.column_position <= cons.col_cnt
  group by i.index_name
  )
  order by table_name, 1
  ;
BEGIN
   dbms_output.put_line('Creating foreign key indexes...');

   FOR stmt_rec in c1
   LOOP
     dbms_output.put_line(stmt_rec.stmt);
     EXECUTE IMMEDIATE stmt_rec.stmt;
   END LOOP;
END;
/

